/**
 * Two simple nodes counting together.
 */
#include <boost/bind.hpp>
#include <boost/thread.hpp>

#include <rsb/Factory.h>

#include "cca/Config.h"
#include "cca/Node.h"
#include "cca/nodes/Collector.h"
#include "cca/port/PortConfiguration.h"
#include "cca/processing/RestlessProcessing.h"
#include "cca/processing/ModerateProcessing.h"

#include "rci/dto/JointAngles.h"
#include "rci/dto/converter/JointAnglesConverter.h"

using namespace cca;
using namespace cca::rci;
using namespace boost;
using namespace rsc;
using namespace rsc::logging;
using namespace rsb;
using namespace rsb::converter;
using namespace rst::converter::rci;

class InNode: public CCANode {
public:
    InNode() :
        CCANode("IN") {

        // Create typed ports, not configured yet
        this->addInputPort<JointAngles> ();
    }
    std::string print() const {
        return "<InNode>";
    }
    void onProcess() {
        std::cout << "[Node " << this->getName() << "]";

        JointAngles foo = *boost::static_pointer_cast<JointAngles>(
                this->inputs[0]->tryGet());

        std::cout << " got value: " << foo << std::endl;
    }
private:
    InNode(InNode &node);
    void operator=(const InNode &node);
};

class OutNode: public CCANode {
public:
    OutNode(double sendValue = 0.11) :
        CCANode("OUT"), value(sendValue) {

        // Create typed ports, not configured yet
        this->addOutputPort<JointAngles> ();
    }
    std::string print() const {
        return "<OutNode>";
    }
    void onProcess() {
        std::cout << "[Node " << this->getName() << "] sending three values"
                << std::endl;

        JointAngles *foo = new JointAngles(value);
        rsb::Informer<JointAngles>::DataPtr out(foo);
        this->publish(0, out);
    }
private:
    OutNode(OutNode &node);
    void operator=(const OutNode &node);
    double value;
};

int main() {
    std::string in = "/example/multidim";
    std::string out0 = "/example/onedim0";
    std::string out1 = "/example/onedim1";
    std::string out2 = "/example/onedim2";

    // Setting up a splitter
    CCANodePtr collector = Collector<JointAngles>::create(3);
    collector->setProcessingStrategy(ModerateProcessing::create());
    collector->configureInputPort(0, PortConfiguration::LOCAL(out0));
    collector->configureInputPort(1, PortConfiguration::LOCAL(out1));
    collector->configureInputPort(2, PortConfiguration::LOCAL(out2));
    collector->configureOutputPort(0, PortConfiguration::LOCAL(in));

    // Setting up receiving node
    CCANodePtr rcvr = CCANodePtr(new InNode());
    rcvr->setProcessingStrategy(RestlessProcessing::create());
    rcvr->configureInputPort(0, PortConfiguration::LOCAL(in));

    // Setting up sending nodes
    CCANodePtr sender0 = CCANodePtr(new OutNode(0.11));
    CCANodePtr sender1 = CCANodePtr(new OutNode(2.12));
    CCANodePtr sender2 = CCANodePtr(new OutNode(3.14));
    sender0->setProcessingStrategy(RestlessProcessing::create());
    sender1->setProcessingStrategy(RestlessProcessing::create());
    sender2->setProcessingStrategy(RestlessProcessing::create());
    sender0->configureOutputPort(0, PortConfiguration::LOCAL(out0));
    sender1->configureOutputPort(0, PortConfiguration::LOCAL(out1));
    sender2->configureOutputPort(0, PortConfiguration::LOCAL(out2));

    sender0->onProcess();
    sender1->onProcess();
    sender2->onProcess();

    sleep(1);

    return EXIT_SUCCESS;
}
