#include <boost/bind.hpp>
#include <boost/thread.hpp>

#include <rsc/logging/LoggerFactory.h>

#include <rsb/Factory.h>
#include <rsb/converter/Repository.h>

#include <cca/Config.h>
#include <cca/Beat.h>
#include <cca/ProcessingStrategy.h>
#include <cca/processing/TimedProcessing.h>

#include "rci/dto/CartesianPose.h"

using namespace cca;
using namespace cca::rci;
using namespace boost;
using namespace rsc;
using namespace rsc::logging;
using namespace rsb;

/**
 * Test Orientation and Rotation
 */
int main() {
    LoggerFactory::getInstance().reconfigure(Logger::LEVEL_WARN);

    OrientationPtr rotationX = Orientation::fromEulerAngles(M_PI, 0.0, 0.0);
    std::cout << "Rotation of PI around X: " << *rotationX;
    std::cout << "Rotation of PI around X: " << rotationX->asEulerAngles() << std::endl << std::endl;

    OrientationPtr rotationY = Orientation::fromEulerAngles(0.0, M_PI, 0.0);
    std::cout << "Rotation of PI around Y: " << *rotationY;
    std::cout << "Rotation of PI around Y: " << rotationY->asEulerAngles() << std::endl << std::endl;

    OrientationPtr rotationZ = Orientation::fromEulerAngles(0.0, 0.0, M_PI);
    std::cout << "Rotation of PI around Z: " << *rotationZ;
    std::cout << "Rotation of PI around Z: " << rotationZ->asEulerAngles() << std::endl << std::endl;

    OrientationPtr rotationXY = rotationX->rotateBy(rotationY);
    std::cout << "Rotation of PI around X then Y: " << *rotationXY;
    std::cout << "Rotation of PI around X then Y: " << rotationXY->asEulerAngles() << std::endl << std::endl;

    OrientationPtr rotationYX = rotationY->rotateBy(rotationX);
    std::cout << "Rotation of PI around Y then X: " << *rotationYX;
    std::cout << "Rotation of PI around Y then X: " << rotationYX->asEulerAngles() << std::endl << std::endl;

    OrientationPtr rotationX2 = Orientation::fromEulerAngles(2* M_PI, 0.0, 0.0);
    std::cout << "Rotation of 2*PI around X: " << *rotationX2;
    std::cout << "Rotation of 2*PI around X: " << rotationX2->asEulerAngles() << std::endl << std::endl;

    return 0;
}
