/**
 * Two simple nodes counting together.
 */
#include <boost/bind.hpp>
#include <boost/thread.hpp>

#include <rsb/Factory.h>

#include "cca/Config.h"
#include "cca/Node.h"
#include "cca/nodes/Splitter.h"
#include "cca/port/PortConfiguration.h"
#include "cca/processing/RestlessProcessing.h"

#include "rci/dto/JointAngles.h"
#include "rci/dto/converter/JointAnglesConverter.h"

using namespace cca;
using namespace cca::rci;
using namespace boost;
using namespace rsc;
using namespace rsc::logging;
using namespace rsb;
using namespace rsb::converter;
using namespace rst::converter::rci;

class InNode: public CCANode {
public:
    InNode() :
        CCANode("IN") {

        // Create typed ports, not configured yet
        this->addInputPort<JointAngles> ();
    }
    std::string print() const {
        return "<InNode>";
    }
    void onProcess() {
        std::cout << "[Node " << this->getName() << "]";

        double foo = boost::static_pointer_cast<JointAngles>(
                this->inputs[0]->tryGet())->asDouble(0);

        std::cout << " got value: " << foo << std::endl;
    }
private:
    InNode(InNode &node);
    void operator=(const InNode &node);
};

class OutNode: public CCANode {
public:
    OutNode() :
        CCANode("OUT") {

        // Create typed ports, not configured yet
        this->addOutputPort<JointAngles> ();
    }
    std::string print() const {
        return "<OutNode>";
    }
    void onProcess() {
        std::cout << "[Node " << this->getName() << "] sending three values"
                << std::endl;

        nemo::RealVector values(nemo::dim(3), 0.0);
        values[0] = 0.11;
        values[1] = 2.22;
        values[2] = 333.3;
        JointAngles *foo = new JointAngles(values);
        rsb::Informer<JointAngles>::DataPtr out(foo);
        this->publish(0, out);
    }
private:
    OutNode(OutNode &node);
    void operator=(const OutNode &node);
};

int main() {
    std::string in = "/example/multidim";
    std::string out0 = "/example/onedim0";
    std::string out1 = "/example/onedim1";
    std::string out2 = "/example/onedim2";

    // Setting up a splitter
    CCANodePtr splitter = Splitter<JointAngles>::create(3);
    splitter->setProcessingStrategy(RestlessProcessing::create());
    splitter->configureInputPort(0, PortConfiguration::LOCAL(in));
    splitter->configureOutputPort(0, PortConfiguration::LOCAL(out0));
    splitter->configureOutputPort(1, PortConfiguration::LOCAL(out1));
    splitter->configureOutputPort(2, PortConfiguration::LOCAL(out2));

    // Setting up receiving nodes
    CCANodePtr in0 = CCANodePtr(new InNode());
    CCANodePtr in1 = CCANodePtr(new InNode());
    CCANodePtr in2 = CCANodePtr(new InNode());
    in0->setProcessingStrategy(RestlessProcessing::create());
    in1->setProcessingStrategy(RestlessProcessing::create());
    in2->setProcessingStrategy(RestlessProcessing::create());
    in0->configureInputPort(0, PortConfiguration::LOCAL(out0));
    in1->configureInputPort(0, PortConfiguration::LOCAL(out1));
    in2->configureInputPort(0, PortConfiguration::LOCAL(out2));

    // Setting up sending node
    CCANodePtr out = CCANodePtr(new OutNode());
    out->setProcessingStrategy(RestlessProcessing::create());
    out->configureOutputPort(0, PortConfiguration::LOCAL(in));

    out->onProcess();

    sleep(1);

    return EXIT_SUCCESS;
}
