#pragma once

#include <cmath>

#include <iostream>
#include <stdexcept>

#include <boost/shared_ptr.hpp>

#include <rsc/runtime/Printable.h>

#include <nemo/Vector.h>

namespace cca {

/**
 * Base for data transfer object, representing domain-specific data.
 * @defgroup DTO DataTransferObjects
 */
class DataTransferObject;
typedef boost::shared_ptr<DataTransferObject> DataTransferObjectPtr;
typedef boost::shared_ptr<void> VoidPtr;

/**
 * Base data transfer object, representing domain-specific data. All inheriting
 * classes should use the International System of Units (SI) as default data
 * representation.
 *
 * @ingroup DTO
 *
 * @todo overload operators (+=, -=, *= (), [])
 * @todo provide friend operators (+, -, *, /, !=, ==)
 */
class DataTransferObject {

public:

    /**
     * Prints the DataTransferObject
     *
     * @return DataTranserObject as String
     */
    virtual std::string print() const {
        std::string str;
        str = "<DataTransferObject>";
        return str;
    }

    /**
     * Implement this method and return a human-readable class name. This method
     * should always return the same class name. Its existence is an admission
     * to missing RTTI functionalities in C++.
     *
     * @return human-readable name of the class.
     */
    //virtual std::string getClassName() const = 0;

    /**
     * This method does the actual printing of class debug information and
     * should be overridden in subclasses.
     *
     * @param stream stream to print on
     */
//    virtual void printContents(std::ostream& stream) const {
//        stream << this->print();
//    }
    
protected:
    /**
     * Constructor
     * Initializes DTO with specified dimension.
     */
    DataTransferObject() {
    }

    virtual ~DataTransferObject() {
    }
};

inline std::ostream & operator<<(std::ostream &os,
        const DataTransferObject& dto) {
    os.precision(3); // Precision when printing double values
    os << dto.print();
    return os;
}

}
