#pragma once

#include <sstream>
#include <iostream>
#include <math.h>

#include <cca/Config.h>
#include <cca/Node.h>

#include <NemoMath/Vector.h>

namespace cca {
namespace rci {

//template<class T>
//class Controller;
//typedef boost::shared_ptr<Controller<T> > ControllerPtr;

/**
 * Simple P-Controller.
 * @todo Make it data-type generic (template).
 */
template<class T>
class Controller: public CCANode {

public:

    Controller(std::string nodename) :
        CCANode(nodename), _task(), _status() {

        this->addInputPort<T> ();
        this->addInputPort<T> ();
        this->addOutputPort<T> ();

        _task = nemo::RealVector(nemo::dim(1), 0.0);
        _status = nemo::RealVector(nemo::dim(1), 0.0);
    }
    virtual ~Controller() {
    }

    /**
     * Printing information about this node
     */
    virtual std::string print() const {
        std::ostringstream outstream(std::ostringstream::out);
        outstream.precision(3); // Precision when printing double values
        outstream << "<Controller \"" << this->getName() << "\">" << std::endl;
        return outstream.str();
    }

    /**
     * This is the method where the actual processing of the node takes place.
     * @param task The task to be processed by this node
     */
    virtual void onProcess() {
        //std::cout << "[Controller] Controlling ..." << std::endl;
        T *step;

        // Take input values
        if (!this->inputs[0]->empty() && this->inputs[0]->newItem()) {
            _task = *boost::static_pointer_cast<T>(this->inputs[0]->tryGet());
            std::cout << "[Controller] New task is\t" << _task;
        }

        if (!this->inputs[1]->empty() && this->inputs[1]->newItem()) {
            _status = *boost::static_pointer_cast<T>(this->inputs[1]->tryGet());
            std::cout << "[Controller] Status is\t\t" << _status;
        }

        // Calculate step
        double dstep = (_task.asDouble(0) - _status.asDouble(0)) / 4;
        step = new T(dstep);
        std::cout << "[Controller] Ctrl step is\t" << *step;

        // Publish task
        typename rsb::Informer<T>::DataPtr task(step);
        this->publish(0, task);
    }

private:
    /**
     * Copy constructor
     * Copy constructor private for now to disallow copying.
     */
    Controller(Controller &node);

    /**
     * Copy operator
     * Copy operator private for now to disallow copying.
     */
    void operator=(const Controller &node);

    T _task, _status;
};

}
}
