#pragma once

#include <iostream>

#include <boost/shared_ptr.hpp>

namespace rci {

/**
 * Helper methods for RCI.
 *
 * @ingroup DTO
 */
class RCIHelper {

public:

    /// Constructor
    RCIHelper(const std::string &name, const std::string &ID);

    /// Destructor
    virtual ~RCIHelper();

    /**
     * Merges several low-dimensional DTOs into one high-dimensional DTO.
     *
     * @param inputDTOs
     *          Vector of low-dimensional input DTOs to merge
     *
     * @return Merged high-dimensional DTO
     */
    template<class DTOType>
    static boost::shared_ptr<DTOType> mergeDTOs(
            const std::vector<boost::shared_ptr<DTOType> > inputDTOs) {
        // Calculate output dimension
        unsigned int outputdimension = 0;
        for (unsigned int inputdto = 0; inputdto < inputDTOs.size();
                ++inputdto) {
            for (unsigned int inputdim = 0;
                    inputdim < inputDTOs[inputdto]->getDimension();
                    ++inputdim) {
                ++outputdimension;
            }
        }

        // Create and return merged DTOs
        boost::shared_ptr<DTOType> merged = DTOType::create(outputdimension);
        RCIHelper::mergeDTOs(merged, inputDTOs);
        return merged;
    }

    /**
     * Merges several low-dimensional DTOs into one high-dimensional DTO.
     *
     * @param outputDTO
     *          High-dimensional output DTO
     * @param inputDTOs
     *          Vector of low-dimensional input DTOs to merge
     */
    template<class DTOType>
    static void mergeDTOs(boost::shared_ptr<DTOType> outputDTO,
            const std::vector<boost::shared_ptr<DTOType> > inputDTOs) {
        unsigned int outputcursor = 0;

        for (unsigned int inputdto = 0; inputdto < inputDTOs.size();
                ++inputdto) {
            for (unsigned int inputdim = 0;
                    inputdim < inputDTOs[inputdto]->getDimension();
                    ++inputdim) {
                outputDTO->setValue(outputcursor++,
                        inputDTOs[inputdto]->asDouble(inputdim));
            }
        }

    }

private:
    RCIHelper(const RCIHelper& RCIHelper);
};

}
