#pragma once

#include <iostream>
#include <sstream>

// RSC
#include <rsc/math/MathUtils.h>

#include "rci/dto/CartesianPose.h"

namespace rci {

class Acceleration;
typedef boost::shared_ptr<Acceleration> AccelerationPtr;
class TranslationalAcceleration;
typedef boost::shared_ptr<TranslationalAcceleration> TranslationalAccelerationPtr;
class RotationalAcceleration;
typedef boost::shared_ptr<RotationalAcceleration> RotationalAccelerationPtr;

/**
 * Domain object, representing a cartesian
 * translatory velocity. Default representation is in \f$\frac{m}{s}\f$,
 * according to the International Unit System (SI).
 * @ingroup DTO
 */
class TranslationalAcceleration: public CartesianTranslational {

public:

    TranslationalAcceleration();
    TranslationalAcceleration(double valx, double valy, double valz);
    TranslationalAcceleration(nemo::RealVector values);
    ~TranslationalAcceleration();

    virtual double m_s2(unsigned int index);
    virtual nemo::RealVector m_s2Vector();

    /**
     * Shared Pointer Factory
     */
    static TranslationalAccelerationPtr fromM_s2(double x, double y, double z);
protected:
    /**
     * Printing the pose object
     * @return Textual representation of this pose.
     */
    virtual std::string print() const;
};

}
