#include "CartesianDamping.h"

using namespace nemo;

namespace rci {

TranslationalDamping::TranslationalDamping() :
    CartesianTranslational() {
}

TranslationalDamping::TranslationalDamping(double valx, double valy, double valz) :
    CartesianTranslational(valx, valy, valz) {
}

TranslationalDamping::TranslationalDamping(const TranslationalDamping& damping) :
    CartesianTranslational() {
    this->_values = RealVector(dim(3));
    this->_values[0] = damping.asDouble(0);
    this->_values[1] = damping.asDouble(1);
    this->_values[2] = damping.asDouble(2);
}

TranslationalDamping::TranslationalDamping(nemo::RealVector values) :
    CartesianTranslational(values) {
}

RotationalDamping::RotationalDamping() :
    CartesianRotational() {
}

RotationalDamping::RotationalDamping(double vala, double valb, double valc, double vald) :
    CartesianRotational(vala, valb, valc, vald) {
}

RotationalDamping::RotationalDamping(RealVector values) :
    CartesianRotational(values) {
}

RotationalDamping::RotationalDamping(const RotationalDamping& damping) :
    CartesianRotational() {
    this->_values = RealVector(damping.asDouble(0), damping.asDouble(1),
            damping.asDouble(2));
}

CartesianDamping::CartesianDamping() :
    CartesianValue() {
}

CartesianDamping::CartesianDamping(double valx, double valy, double valz,
        double vala, double valb, double valc, double vald) :
    CartesianValue(valx, valy, valz, vala, valb, valc, vald) {
}

CartesianDamping::CartesianDamping(const CartesianDamping& damping) :
    CartesianValue(damping) {
}

CartesianDamping::CartesianDamping(RealVector values) :
    CartesianValue(values) {
}

}
