#pragma once

#include <iostream>

#include "rci/dto/CartesianValues.h"

namespace rci {

/**
 * Domain object, representing a cartesian force and rotatory stiffness in all
 * three dimensions. The default unit is newton per meter (\f$N/m\f$).
 * @ingroup DTO
 * @ingroup Compliance
 */
class TranslationalDamping: public CartesianTranslational {

public:
    TranslationalDamping();
    TranslationalDamping(double x, double y, double z);
    TranslationalDamping(nemo::RealVector values);
    TranslationalDamping(const TranslationalDamping& damping);
};

/**
 * Domain object, representing an orientation or rotational transformation in
 * cartesian space. The default unit is newton-metres per radian
 * (\f$N*m/rad\f$).
 * @ingroup DTO
 * @ingroup Compliance
 */
class RotationalDamping: public CartesianRotational {

public:
    RotationalDamping();
    RotationalDamping(double vala, double valb, double valc, double vald);
    RotationalDamping(nemo::RealVector values);
    RotationalDamping(const RotationalDamping& damping);
};

/**
 * Domain object, representing a full cartesian pose, consisting of
 * translational transformation and orientation.
 * @ingroup DTO
 * @ingroup Compliance
 */
class CartesianDamping: public CartesianValue {

public:
    CartesianDamping();
    CartesianDamping(double valx, double valy, double valz, double vala,
            double valb, double valc, double vald);
    CartesianDamping(nemo::RealVector values);
    CartesianDamping(const CartesianDamping& damping);
};

}
