#pragma once

#include <iostream>
#include <boost/shared_ptr.hpp>

#include "rci/dto/CartesianValues.h"

namespace rci {

class Forces;
typedef boost::shared_ptr<Forces> ForcesPtr;

/**
 * Domain object, representing a cartesian force and torques in all three
 * dimensions. Default units are (N, N, N, N*cm, N*cm, N*cm).
 * @ingroup DTO
 */
class Forces: public CartesianTranslational {

public:
    Forces();
    Forces(double x, double y, double z);
    Forces(nemo::RealVector values);
    Forces(const Forces& forces);
};

/**
 * Domain object, representing an orientation or rotational transformation in
 * cartesian space.
 *
 * @ingroup DTO
 */
class Torques: public CartesianRotational {

public:
    Torques();
    Torques(double vala, double valb, double valc, double vald);
    Torques(nemo::RealVector values);
    Torques(const Torques& torques);
};

/**
 * Domain object, representing a full cartesian pose, consisting of
 * translational transformation and orientation.
 * @ingroup DTO
 */
class Interaction: public CartesianValue {

public:
    Interaction();
    Interaction(double valx, double valy, double valz, double vala,
            double valb, double valc, double vald);
    Interaction(nemo::RealVector values);
    Interaction(const Interaction& intact);
};

}
