#include "CartesianImpedance.h"

using namespace nemo;

namespace rci {

TranslationalImpedance::TranslationalImpedance() :
        CartesianTranslational() {
}

TranslationalImpedance::TranslationalImpedance(double valx, double valy,
        double valz) :
        CartesianTranslational(valx, valy, valz) {
}

TranslationalImpedance::TranslationalImpedance(
        const TranslationalImpedance& impedance) :
        CartesianTranslational() {
    this->_values = RealVector(dim(3));
    this->_values[0] = impedance.asDouble(0);
    this->_values[1] = impedance.asDouble(1);
    this->_values[2] = impedance.asDouble(2);
}

TranslationalImpedance::TranslationalImpedance(nemo::RealVector values) :
        CartesianTranslational(values) {
}

RotationalImpedance::RotationalImpedance() :
        CartesianRotational() {
}

RotationalImpedance::RotationalImpedance(double vala, double valb, double valc,
        double vald) :
        CartesianRotational(vala, valb, valc, vald) {
}

RotationalImpedance::RotationalImpedance(nemo::RealVector values) :
        CartesianRotational(values) {
}

RotationalImpedance::RotationalImpedance(const RotationalImpedance& impedance) :
        CartesianRotational() {
    this->_values = RealVector(impedance.asDouble(0), impedance.asDouble(1),
            impedance.asDouble(2));
}

CartesianImpedance::CartesianImpedance() :
        CartesianValue() {
}

CartesianImpedance::CartesianImpedance(double valx, double valy, double valz,
        double vala, double valb, double valc, double vald) :
        CartesianValue(valx, valy, valz, vala, valb, valc, vald) {
}

CartesianImpedance::CartesianImpedance(const CartesianImpedance& impedance) :
        CartesianValue(impedance) {
}

CartesianImpedance::CartesianImpedance(nemo::RealVector values) :
        CartesianValue(values) {
    this->_transl = TranslationalImpedance(values[0], values[1], values[2]);
    this->_rot = RotationalImpedance(values[3], values[4], values[5],
            values[6]);
}

}
