#pragma once

#include <iostream>

#include "rci/dto/CartesianValues.h"

namespace rci {

/**
 * Compliance bla
 * @defgroup Compliance Compliance
 */

/**
 * Domain object, representing a cartesian force and RotationalImpedance in all three
 * dimensions. Default units are (N, N, N, N*cm, N*cm, N*cm).
 *
 * @ingroup DTO
 * @ingroup Compliance
 */
class TranslationalImpedance: public CartesianTranslational {

public:
    TranslationalImpedance();
    TranslationalImpedance(double x, double y, double z);
    TranslationalImpedance(nemo::RealVector values);
    TranslationalImpedance(const TranslationalImpedance& impedance);
};

/**
 * Domain object, representing an orientation or rotational transformation in
 * cartesian space.
 *
 * @ingroup DTO
 * @ingroup Compliance
 */
class RotationalImpedance: public CartesianRotational {

public:
    RotationalImpedance();
    RotationalImpedance(double vala, double valb, double valc, double vald);
    RotationalImpedance(nemo::RealVector values);
    RotationalImpedance(const RotationalImpedance& impedance);
};

/**
 * Domain object, representing a full cartesian pose, consisting of
 * translational transformation and orientation.
 * @ingroup DTO
 * @ingroup Compliance
 */
class CartesianImpedance: public CartesianValue {

public:
    CartesianImpedance();
    CartesianImpedance(double valx, double valy, double valz, double vala,
            double valb, double valc, double vald);
    CartesianImpedance(nemo::RealVector values);
    CartesianImpedance(const CartesianImpedance& impedance);
};

}
