#include "CartesianStiffness.h"

using namespace nemo;

namespace rci {

TranslationalStiffness::TranslationalStiffness() :
    CartesianTranslational() {
}

TranslationalStiffness::TranslationalStiffness(double valx, double valy,
        double valz) :
    CartesianTranslational(valx, valy, valz) {
}

TranslationalStiffness::TranslationalStiffness(const TranslationalStiffness& transl) :
    CartesianTranslational() {
    this->_values = RealVector(dim(3));
    this->_values[0] = transl.asDouble(0);
    this->_values[1] = transl.asDouble(1);
    this->_values[2] = transl.asDouble(2);
}

TranslationalStiffness::TranslationalStiffness(nemo::RealVector values) :
    CartesianTranslational(values) {
}

RotationalStiffness::RotationalStiffness() :
    CartesianRotational() {
}

RotationalStiffness::RotationalStiffness(double vala, double valb, double valc, double vald) :
    CartesianRotational(vala, valb, valc, vald) {
}

RotationalStiffness::RotationalStiffness(nemo::RealVector values) :
    CartesianRotational(values) {
}

RotationalStiffness::RotationalStiffness(const RotationalStiffness& rot) :
    CartesianRotational() {
    this->_values = RealVector(rot.asDouble(0), rot.asDouble(1),
            rot.asDouble(2));
}

CartesianStiffness::CartesianStiffness() :
    CartesianValue() {
}

CartesianStiffness::CartesianStiffness(double valx, double valy, double valz,
        double vala, double valb, double valc, double vald) :
    CartesianValue(valx, valy, valz, vala, valb, valc, vald) {
}

CartesianStiffness::CartesianStiffness(const CartesianStiffness& intact) :
    CartesianValue(intact) {
}

CartesianStiffness::CartesianStiffness(nemo::RealVector values) :
    CartesianValue(values) {
}

}
