#pragma once

#include <iostream>

#include "rci/dto/CartesianValues.h"

namespace rci {

/**
 * Domain object, representing a cartesian force and rotatory stiffness in all
 * three dimensions. The default unit is newton per meter (\f$N/m\f$).
 * @ingroup DTO
 * @ingroup Compliance
 */
class TranslationalStiffness: public CartesianTranslational {

public:
    TranslationalStiffness();
    TranslationalStiffness(double x, double y, double z);
    TranslationalStiffness(nemo::RealVector values);
    TranslationalStiffness(const TranslationalStiffness& stiffness);
};

/**
 * Domain object, representing an orientation or rotational transformation in
 * cartesian space. The default unit is newton-metres per radian
 * (\f$N*m/rad\f$).
 * @ingroup DTO
 * @ingroup Compliance
 */
class RotationalStiffness: public CartesianRotational {

public:
    RotationalStiffness();
    RotationalStiffness(double a, double b, double c, double d);
    RotationalStiffness(nemo::RealVector values);
    RotationalStiffness(const RotationalStiffness& RotationalStiffness);
};

/**
 * Domain object, representing a full cartesian pose, consisting of
 * translational transformation and orientation.
 * @ingroup DTO
 * @ingroup Compliance
 */
class CartesianStiffness: public CartesianValue {

public:
    CartesianStiffness();
    CartesianStiffness(double x, double y, double z, double a, double b,
            double c, double d);
    CartesianStiffness(nemo::RealVector values);
    CartesianStiffness(const CartesianStiffness& intact);
};

}
