#include "JointTorques.h"

using namespace std;
using namespace nemo;

namespace rci {

JointTorquesPtr JointTorques::fromNm(double value) {
    return JointTorquesPtr(new JointTorques(RealVector(value)));
}

JointTorquesPtr JointTorques::fromNm(const nemo::RealVector& values) {
    return JointTorquesPtr(new JointTorques(values));
}

JointTorques::JointTorques() :
        JointValues() {
}

JointTorques::JointTorques(double value) :
        JointValues(value) {
}

JointTorques::JointTorques(const RealVector& values) :
        JointValues(values) {
}

JointTorquesPtr JointTorques::create(unsigned int dimension, double value) {
    RealVector values = RealVector(dim(dimension), value);
    return JointTorques::fromNm(values);
}

double JointTorques::Nm(unsigned int index) const {
    if (index >= _values.dimension()) {
        std::stringstream errMsg;
        errMsg << "[JointTorques::Nm] index=" << index << " out of bounds 0.."
                << _values.dimension() << "!";
        throw std::runtime_error(errMsg.str());
    }
    return _values[index];
}

nemo::RealVector JointTorques::NmVector() const {
    return _values;
}

std::string JointTorques::print() const {
    ostringstream outstream(ostringstream::out);
    outstream.precision(3); // Precision when printing double values
    outstream << "JointTorques<";
    for (unsigned int i = 0; i < this->getDimension(); ++i) {
        if (i > 0) {
            outstream << ",";
        }
        outstream << this->asDouble(i);
    }
    outstream << ">(N*cm)" << std::endl;
    return outstream.str();
}

void JointTorques::setFromNm(unsigned int index, double value) {
    if (index >= _values.dimension()) {
        std::stringstream errMsg;
        errMsg << "[JointTorques::setFromNm] index=" << index
                << " out of bounds 0.." << _values.dimension() << "!";
        throw std::runtime_error(errMsg.str());
    }
    _values[index] = value;
}

void JointTorques::setFromNm(const nemo::RealVector& values) {
    if (values.dimension() != _values.dimension()) {
        std::stringstream errMsg;
        errMsg << "[JointTorques::setFromNm] Dimension mismatch! Expected dim="
                << _values.dimension() << "  but got dim=" << values.dimension()
                << "!";
        throw std::runtime_error(errMsg.str());
    }
    _values = values;
}

JointTorquesPtr JointTorques::copy(const JointTorques& torques) {
    return JointTorquesPtr(new JointTorques(torques));
}

}
