#include "JointVelocities.h"

using namespace std;
using namespace nemo;
using namespace cca;

namespace rci {

JointVelocities::JointVelocities() :
        JointValues() {
}

JointVelocities::JointVelocities(RealVector values) :
        JointValues(values) {
}

JointVelocities::JointVelocities(double value) :
        JointValues(value) {
}

std::string JointVelocities::print() const {
    ostringstream outstream(ostringstream::out);
    outstream.precision(3); // Precision when printing double values
    outstream << "JointVelocities<";
    for (unsigned int i = 0; i < this->getDimension(); ++i) {
        if (i > 0) {
            outstream << ",";
        }
        outstream << this->asDouble(i);
    }
    outstream << ">(rad)" << std::endl;
    return outstream.str();
}

JointVelocitiesPtr JointVelocities::fromValue(double value) {
    nemo::RealVector v = RealVector(value);
    return JointVelocitiesPtr(new JointVelocities(value));
}

JointVelocitiesPtr JointVelocities::fromValues(RealVector values) {
    return JointVelocitiesPtr(new JointVelocities(values));
}

JointVelocitiesPtr JointVelocities::fromRad_s(double value) {
    return JointVelocities::fromValue(value); // rad/s is default representation
}

JointVelocitiesPtr JointVelocities::fromRad_s(RealVector values) {
    return JointVelocities::fromValues(values); // rad/s is default representation
}

nemo::RealVector JointVelocities::rad_sVector() {
    return this->asDoubleVector();
}

double JointVelocities::rad_s(unsigned int index) {
    return this->asDouble(index);
}

JointVelocitiesPtr JointVelocities::fromDeg_s(double value) {
    return JointVelocities::fromRad_s(value / 180 * M_PI);
}

JointVelocitiesPtr JointVelocities::fromDeg_s(RealVector values) {
    Mapping<double, double> degToRad = DegToRad::create();
    values.transform(degToRad);

    return JointVelocities::fromRad_s(values);
}

nemo::RealVector JointVelocities::deg_sVector() {
    Mapping<double, double> radToDeg = RadToDeg::create();

    return this->_values.map(radToDeg);
}

double JointVelocities::deg_s(unsigned int index) {
    return this->rad_s(index) / M_PI * 180;
}

}
