#pragma once

#include <iostream>
#include <sstream>
#include <math.h>

#include <boost/shared_ptr.hpp>

// RSC
#include <rsc/math/MathUtils.h>

#include <cca/dto/DataTransferObject.h>
#include "cca/dto/ContainingDoubles.h"

namespace rci {

class Length;
typedef boost::shared_ptr<Length> LengthPtr;

/**
 * Domain object, representing length values.
 * Default representatin is in meter (m), according to the International
 * System of Units (SI).
 * @ingroup DTO
 */
class Length: public cca::DataTransferObject, public cca::ContainingDoubles {

public:
    Length();
    Length(nemo::RealVector values);

    /**
     * Returns length in meters.
     *
     * @param index
     * 			Index of the value to return
     * @return Length in meter
     */
    virtual double m(unsigned int index) const;

    /**
     * Create JointAngles from a JointAngles object (deep copy)
     *
     * @param JointAngles object
     */
    static LengthPtr copy(const Length& length);

protected:
    /**
     * Print function
     */
    virtual std::string print() const;
};

}
