#pragma once

#include <iostream>
#include <sstream>
#include <math.h>
#include <boost/shared_ptr.hpp>

#include <cca/dto/DataTransferObject.h>
#include "cca/dto/ContainingDoubles.h"

namespace rci {

class Pressure;
typedef boost::shared_ptr<Pressure> PressurePtr;

/**
 * Domain object, representing pressure values.
 * Default representatin is in pascal (Pa), according to the International
 * System of Units (SI).
 * @ingroup DTO
 */
class Pressure: public cca::DataTransferObject, public cca::ContainingDoubles {

public:
    Pressure();
    Pressure(nemo::RealVector values);

    /**
     * Returns pressure in pascal.
     *
     * @param index
     * 			Index of the value to return
     * @return Pressure in pascal
     */
    virtual double Pa(unsigned int index) const;

    /**
     * Create Pressure from a Pressure object (deep copy)
     *
     * @param Pressure object
     */
    static PressurePtr copy(const Pressure& pressure);

protected:
    /**
     * Print function
     */
    virtual std::string print() const;
};

}
