#include <gtest/gtest.h>

#include <iostream>
#include <vector>
#include <boost/shared_ptr.hpp>

#include "rci/dto/CartesianForces.h"

using namespace nemo;
using namespace rci;

class InteractionTest: public ::testing::Test {
protected:

    InteractionTest() :
            values(), intact() {
        values = RealVector(dim(7));
        values[0] = 1.12;
        values[1] = 2.0;
        values[2] = -1.1;
        values[3] = 1.0;
        values[4] = 34.0;
        values[5] = 0.0;
        values[6] = 0.0;
        intact = Interaction(values);
    }

    virtual ~InteractionTest() {
    }

    RealVector values;
    Interaction intact;
};

TEST_F(InteractionTest, testInitialization)
{
    EXPECT_EQ(7, Interaction().getDimension());
}

TEST_F(InteractionTest, testConstruction)
{
    // Check dimension
    Interaction i = Interaction(values);
    EXPECT_EQ(7, i.getDimension());

    // Check dimension
    i = Interaction(1, 2, 3, 4, 5, 6,7);
    EXPECT_EQ(7, i.getDimension());
}

TEST_F(InteractionTest, testValueGetter)
{
    RealVector testvalues;

    // Check example values
    EXPECT_DOUBLE_EQ(1.12, intact.asDouble(0));
    EXPECT_DOUBLE_EQ(-1.1, intact.asDouble(2));
    EXPECT_DOUBLE_EQ(34.0, intact.asDouble(4));

    // Check values vector
    testvalues = intact.asDoubleVector();
    EXPECT_DOUBLE_EQ(values[1], testvalues[1]);
    EXPECT_DOUBLE_EQ(values[3], testvalues[3]);
    EXPECT_DOUBLE_EQ(values[5], testvalues[5]);
}

TEST_F(InteractionTest, testCartesianGetter)
{
    // Check position values
    EXPECT_DOUBLE_EQ(1.12, intact.x());
    EXPECT_DOUBLE_EQ(2.0, intact.y());
    EXPECT_DOUBLE_EQ(-1.1, intact.z());

    // Check orientation values
    EXPECT_DOUBLE_EQ(1.00, intact.q0());
    EXPECT_DOUBLE_EQ(34.0, intact.q1());
    EXPECT_DOUBLE_EQ(0.0, intact.q2());
    EXPECT_DOUBLE_EQ(0.0, intact.q3());
}
