#include <gtest/gtest.h>

#include <iostream>
#include <math.h>
#include <boost/shared_ptr.hpp>

#include "rci/dto/JointImpedance.h"

using namespace nemo;
using namespace rci;

class JointDampingTest: public ::testing::Test {
protected:

    JointDampingTest() :
            values(), jd() {
        values = RealVector(dim(3));
        values[0] = 1.2 * M_PI;
        values[1] = -3 * M_PI;
        values[2] = -2 * M_PI;
        jd = JointDamping(values);
    }

    virtual ~JointDampingTest() {
    }

    RealVector values;
    JointDamping jd;
};

TEST_F(JointDampingTest, isEmptyInitially)
{
    EXPECT_EQ(0, JointDamping().getDimension());
}

TEST_F(JointDampingTest, testConstruction)
{
    // Check dimension
    JointDamping j = JointDamping(-1.2);
    EXPECT_EQ(1, j.getDimension());

    // Check dimension
    j = JointDamping(values);
    EXPECT_EQ(3, j.getDimension());
}

TEST_F(JointDampingTest, testGetters)
{
    RealVector testvalues;

    // Check double getters
    EXPECT_DOUBLE_EQ(1.2 * M_PI, jd.asDouble(0));
    EXPECT_DOUBLE_EQ(-3 * M_PI, jd.asDouble(1));
    EXPECT_DOUBLE_EQ(-2 * M_PI, jd.asDouble(2));

    // Check operator[]
    EXPECT_DOUBLE_EQ(1.2 * M_PI, jd[0].asDouble(0));
    EXPECT_DOUBLE_EQ(-3 * M_PI, jd[1].asDouble(0));
    EXPECT_DOUBLE_EQ(-2 * M_PI, jd[2].asDouble(0));

    // Check vector getter
    testvalues = jd.asDoubleVector();
    EXPECT_DOUBLE_EQ(values[0], testvalues[0]);
    EXPECT_DOUBLE_EQ(values[1], testvalues[1]);
    EXPECT_DOUBLE_EQ(values[2], testvalues[2]);
}
