#include <gtest/gtest.h>

#include <iostream>
#include <boost/shared_ptr.hpp>

#include "rci/dto/JointTorques.h"

using namespace nemo;
using namespace rci;

class JointTorquesTest: public ::testing::Test {
protected:

    JointTorquesTest() :
            values(), jt() {
        values = RealVector(dim(3));
        values[0] = 1.2 * M_PI;
        values[1] = -3 * M_PI;
        values[2] = -2 * M_PI;
        jt = JointTorques(values);
    }

    virtual ~JointTorquesTest() {
    }

    RealVector values;
    JointTorques jt;
};

TEST_F(JointTorquesTest, isEmptyInitially)
{
    EXPECT_EQ(0, JointTorques().getDimension());
}

TEST_F(JointTorquesTest, testConstruction)
{
    // Check dimension
    JointTorques j = JointTorques(values);
    EXPECT_EQ(3, j.getDimension());
}

TEST_F(JointTorquesTest, testCopyContructor)
{
    JointTorques cp(jt);
    EXPECT_EQ(jt.getDimension(), cp.getDimension());
    EXPECT_DOUBLE_EQ(jt.asDouble(0), cp.asDouble(0));
    EXPECT_DOUBLE_EQ(jt.asDouble(1), cp.asDouble(1));
    EXPECT_DOUBLE_EQ(jt.asDouble(2), cp.asDouble(2));
}

TEST_F(JointTorquesTest, testStaticCopyCreation)
{
    JointTorquesPtr jtPtr = JointTorques::copy(jt);
    EXPECT_EQ(jtPtr->getDimension(), jt.getDimension());
    EXPECT_DOUBLE_EQ(jt.asDouble(0), jtPtr->asDouble(0));
    EXPECT_DOUBLE_EQ(jt.asDouble(1), jtPtr->asDouble(1));
    EXPECT_DOUBLE_EQ(jt.asDouble(2), jtPtr->asDouble(2));
}

TEST_F(JointTorquesTest, testValues)
{
    RealVector testvalues;

    // Check example values
    EXPECT_DOUBLE_EQ(1.2 * M_PI, jt.asDouble(0));
    EXPECT_DOUBLE_EQ(-3 * M_PI, jt.asDouble(1));
    EXPECT_DOUBLE_EQ(-2 * M_PI, jt.asDouble(2));

    // Check values vector
    testvalues = jt.asDoubleVector();
    EXPECT_DOUBLE_EQ(values[0], testvalues[0]);
    EXPECT_DOUBLE_EQ(values[1], testvalues[1]);
    EXPECT_DOUBLE_EQ(values[2], testvalues[2]);
}

