#include <gtest/gtest.h>

#include <iostream>
#include <math.h>
#include <boost/shared_ptr.hpp>

#include "rci/dto/JointAngles.h"
#include "rci/RCIHelper.h"

using namespace nemo;
using namespace std;
using namespace cca;
using namespace rci;

class MergeDTOTest: public ::testing::Test {
protected:

    MergeDTOTest() :
            values1(dim(2), 1.11), values2(dim(3), 2.22), values3(dim(4), 3.33), onedim1(), onedim2(), onedim3(), multidim1(), multidim2(), multidim3() {
        onedim1 = JointAngles::fromRad(1.11);
        onedim2 = JointAngles::fromRad(2.22);
        onedim3 = JointAngles::fromRad(3.33);

        multidim1 = JointAngles::fromRad(values1);

        values2[2] = 2.345;
        multidim2 = JointAngles::fromRad(values2);

        values3[3] = 3.456;
        multidim3 = JointAngles::fromRad(values3);
    }

    virtual ~MergeDTOTest() {
    }

    RealVector values1, values2, values3;
    JointAnglesPtr onedim1, onedim2, onedim3;
    JointAnglesPtr multidim1, multidim2, multidim3;
};

TEST_F(MergeDTOTest, testMergingOneDimensionalDTOsIntoHighDimensionalDTO)
{
    // Vector of DTOs
    std::vector<JointAnglesPtr> inputs = std::vector<JointAnglesPtr>();
    inputs.push_back(onedim1);
    inputs.push_back(onedim2);
    inputs.push_back(onedim3);

    JointAnglesPtr merged = JointAngles::create(3, 0.0);
    RCIHelper::mergeDTOs(merged, inputs);

    EXPECT_EQ(3, merged->getDimension());

    EXPECT_DOUBLE_EQ(1.11, merged->rad(0));
    EXPECT_DOUBLE_EQ(2.22, merged->rad(1));
    EXPECT_DOUBLE_EQ(3.33, merged->rad(2));
}

TEST_F(MergeDTOTest, testCreatingFromOneDimensionalDTOs)
{
    // Vector of DTOs
    std::vector<JointAnglesPtr> inputs = std::vector<JointAnglesPtr>();
    inputs.push_back(onedim1);
    inputs.push_back(onedim2);
    inputs.push_back(onedim3);

    JointAnglesPtr merged = RCIHelper::mergeDTOs(inputs);

    EXPECT_EQ(3, merged->getDimension());

    EXPECT_DOUBLE_EQ(1.11, merged->rad(0));
    EXPECT_DOUBLE_EQ(2.22, merged->rad(1));
    EXPECT_DOUBLE_EQ(3.33, merged->rad(2));
}

TEST_F(MergeDTOTest, testMergingMultiDimensionalDTOsIntoHighDimensionalDTO)
{
    // Vector of DTOs
    std::vector<JointAnglesPtr> inputs = std::vector<JointAnglesPtr>();
    inputs.push_back(multidim1);
    inputs.push_back(multidim2);
    inputs.push_back(multidim3);

    JointAnglesPtr merged = JointAngles::create(9);
    RCIHelper::mergeDTOs(merged, inputs);

    EXPECT_EQ(9, merged->getDimension());

    EXPECT_DOUBLE_EQ(1.11, merged->rad(0));
    EXPECT_DOUBLE_EQ(1.11, merged->rad(1));
    EXPECT_DOUBLE_EQ(2.22, merged->rad(2));
    EXPECT_DOUBLE_EQ(2.345, merged->rad(4));
    EXPECT_DOUBLE_EQ(3.33, merged->rad(5));
    EXPECT_DOUBLE_EQ(3.456, merged->rad(8));
}

TEST_F(MergeDTOTest, testCreatingFromMultiDimensionalDTOs)
{
    // Vector of DTOs
    std::vector<JointAnglesPtr> inputs = std::vector<JointAnglesPtr>();
    inputs.push_back(multidim1);
    inputs.push_back(multidim2);
    inputs.push_back(multidim3);

    JointAnglesPtr merged = RCIHelper::mergeDTOs(inputs);

    EXPECT_EQ(9, merged->getDimension());

    EXPECT_DOUBLE_EQ(1.11, merged->rad(0));
    EXPECT_DOUBLE_EQ(1.11, merged->rad(1));
    EXPECT_DOUBLE_EQ(2.22, merged->rad(2));
    EXPECT_DOUBLE_EQ(2.345, merged->rad(4));
    EXPECT_DOUBLE_EQ(3.33, merged->rad(5));
    EXPECT_DOUBLE_EQ(3.456, merged->rad(8));
}
