#include <gtest/gtest.h>

#include <iostream>
#include <math.h>
#include <boost/shared_ptr.hpp>

#include "rci/dto/Pressure.h"

using namespace std;
using namespace nemo;
using namespace rci;

class PressureTest: public ::testing::Test {
protected:

    PressureTest() :
            values(), pressure() {
        values = RealVector(dim(6));
        values[0] = 1.12;
        values[1] = 2.0;
        values[2] = -1.1;
        values[3] = 5 * M_PI;
        values[4] = 0.0;
        values[5] = -3 * M_PI;
        pressure = Pressure(values);
    }

    virtual ~PressureTest() {
    }

    RealVector values;
    Pressure pressure;
};

TEST_F(PressureTest, testConstruction)
{
    // Check dimension
    Pressure p = Pressure();
    EXPECT_EQ(0, p.getDimension());

    // Check dimension
    p = Pressure(values);
    EXPECT_EQ(6, p.getDimension());
}

TEST_F(PressureTest, testValueGetter)
{
    RealVector testvalues;

    // Check example values
    EXPECT_DOUBLE_EQ(1.12, pressure.asDouble(0));
    EXPECT_DOUBLE_EQ(-1.1, pressure.asDouble(2));
    EXPECT_DOUBLE_EQ(0, pressure.asDouble(4));

    // Check values vector
    testvalues = pressure.asDoubleVector();
    EXPECT_DOUBLE_EQ(values[1], testvalues[1]);
    EXPECT_DOUBLE_EQ(values[3], testvalues[3]);
    EXPECT_DOUBLE_EQ(values[5], testvalues[5]);
}
